/*
  Use this code template to test communication between your Arduino and the Arduino Joystick app.  You will have to fill in some of the gaps for it to work.
*/
//--------------------------Bluetooth Module----------------------
#include <SoftwareSerial.h> //Include the Serial Library

SoftwareSerial mySerial(12, 2); // TX, RX  Replace "pin" with the digital pin you plan to connect to.
//-----------------------end Bluetooth Module---------------------

//---------------------------Motor--------------------------------
// Left Motor
#define ENA   6
#define IN1   10
#define IN2   11

// Right Motor
#define ENB   5
#define IN3   9
#define IN4   8
//------------------------end Motor-------------------------------

//---------------------LED / voltage montior----------------------
//include the Fast LED library to control the WS2811 LED
#include <FastLED.h>

//define Pin and LED number for sake of convience and also for program to function
#define LED_PIN     13
#define NUM_LEDS    1
#define BRIGHTNESS  64
#define LED_TYPE    WS2811
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];

//define Pin for the voltage divider
#define VOLT_DIV    A5

//define the color palate and blending palate to be used for LED
CRGBPalette16 currentPalette;
TBlendType    currentBlending;

//define variables for LED
int voltage;
int LEDflip;
//-------------------------end LED / voltage monitor----------------------

//no Delay variable
long int preMillis[10];
bool delayBuffer[10];

void setup() {
  //-------------------Motor setup--------------------------
  // Set all the motor control pins to outputs
  pinMode(ENA, OUTPUT);
  pinMode(ENB, OUTPUT);
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  //----------------Motor setup end-------------------------

  //------------LED / voltage monitor setup-----------------
  //setup the WS2811 LED
  FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip );
  FastLED.setBrightness(  BRIGHTNESS );

  //define palate and blending of the WS2811 LED
  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;

  //setup the analog refrence for the voltage divider
  analogReference(EXTERNAL);
  //-----------LED / voltage monitor end--------------------

  Serial.begin(38400); // Fill in the empty parenthesis with a baud rate.
  Serial.println("Testing Robot Car Bluetooth Module and Joystick App");
  mySerial.begin(38400); // Fill in the empty parenthesis with a baud rate.
  //Speed(.5,.5, 100); //Setup Speed
  mySerial.setTimeout(100); //VERY important for fast response
}

void loop()
{
  batteryLED(20); //function for running the LED / voltage monitor

  //wait for Serial moitor to become availible
  if (mySerial.available())
  {
    String value = mySerial.readStringUntil('#');
    if (value.length() == 7)
    {
      //variables  to store values of angle and magnitude
      int angle = (value.substring(0, 3)).toInt();
      int mag = (value.substring(3, 6)).toInt();

      // angle / direction detection
      if (angle < 180)
      {
        if (angle < 90)
        {
          Speed(255, map(angle, 0, 90, -255, 255), mag);
        }
        else
        {
          Speed(map(angle, 90, 180, 255, -255), 255, mag);
        }
      }
      else
      {
        if (angle < 270)
        {
          Speed(-255, map(angle, 180, 270, 255, -255), mag);
        }
        else
        {
          Speed(map(angle, 270, 360, -255, 255), -255, mag);
        }
      }
    }
  }
}

//--------------------------Motor functions-----------------------------
//-------speed functions
//function for dynamic speed adjustment
void Speed(float lSpeed, float rSpeed, float mag) {
  //checks to make sure that speed isn't too low
  if (mag > 30)
  {
    if (rSpeed > 0)
    {
      //set left wheel to drive forward
      digitalWrite(IN1, HIGH);
      digitalWrite(IN2, LOW);
    }
    else
    {
      //set left wheel to drive backward
      digitalWrite(IN1, LOW);
      digitalWrite(IN2, HIGH);
    }
    if (lSpeed > 0)
    {
      //set right wheel to drive forward
      digitalWrite(IN3, HIGH);
      digitalWrite(IN4, LOW);
    }
    else
    {
      //set right wheel to drive backward
      digitalWrite(IN3, LOW);
      digitalWrite(IN4, HIGH);
    }
    analogWrite(ENA, (abs(rSpeed) * (mag / 100)) * .5);
    analogWrite(ENB, (abs(lSpeed) * (mag / 100)) * .5);
  }
  else
  {                                        
    //if speed is below 30, just stop the bot
    //set wheels  to stop
    digitalWrite(IN1, LOW);
    digitalWrite(IN2, LOW);
    digitalWrite(IN3, LOW);
    digitalWrite(IN4, LOW);
  }
}
//-----------------------Motor functions end----------------------------

//------------------LED / voltage moniter functions---------------------
//function that runs the WS2811 with the battery X times a second
void batteryLED(int UPDATES_PER_SECOND)
{
  //use millis to determine how often to sample voltage based on UPDATES_PER_SECOND
  if (millis() - preMillis[3] >= (1000 / UPDATES_PER_SECOND))
  {
    preMillis[3] = millis();

    //funciton for mapping values from voltage divider to actual battery voltages
    voltage = map(analogRead(VOLT_DIV), 0, 1023, 0, 3320) * (8.4 / 3.17);

    //fucntion for displaying LED color depending on voltage
    if (voltage < 6600)
    {
      //if the voltage is below 6.6V, blink LED RED at 5Hz
      if (LEDflip < UPDATES_PER_SECOND / 20)
      {
        leds[0] = ColorFromPalette(currentPalette, 10, 255, currentBlending);
        LEDflip++;
      }
      else
      {
        leds[0] = ColorFromPalette(currentPalette, 10, 0, currentBlending);
        LEDflip++;
        if (LEDflip > 2 * (UPDATES_PER_SECOND / 5))
        {
          LEDflip = 0;
        }

      }
    }
    //if the voltage is between 6.6V and 7.0V, display gradient of yellow based on voltage
    else if (voltage < 7000)
    {
      leds[0] = ColorFromPalette(currentPalette, map(voltage, 6500, 7100, 30, 70), 255, currentBlending);
    }
    //if voltage is above 7.0V, display gradient of green based on voltage
    else
    {
      leds[0] = ColorFromPalette(currentPalette, map(voltage, 6900, 8600, 80, 100), 255, currentBlending);
    }
    FastLED.show();
  }
}
//-----------------------LED / voltage monitor end-----------------------------
